<?php

namespace FiloBlu\ExtEbizmartsMailChimp\Helper;

/**
 * Class Data
 * @package FiloBlu\ExtEbizmartsMailChimp\Helper
 */
class Data extends \Ebizmarts\MailChimp\Helper\Data
{
    /**
     * @var string
     */
    const XML_PATH_CRON = 'mailchimp/general/enable_cron';
    /**
     * @var string
     */
    const XML_PATH_CRON_SYNC_STATUS = 'mailchimp/general/enable_cron_sync_status';
    /**
     * @var \Magento\Customer\Model\ResourceModel\CustomerRepositoryFactory
     */
    protected $_customerRepositoryFactory;
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var \FiloBlu\Newsletter\Model\SubscriberInfoFactory
     */
    protected $_subscriberInfoFactory;
    /**
     * @var \Magento\Customer\Api\GroupRepositoryInterface
     */
    protected $_groupRepository;
    /**
     * @var \Magento\Newsletter\Model\SubscriberFactory
     */
    protected $_subscriberFactory;

    /**
     * Data constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Ebizmarts\MailChimp\Model\Logger\Logger $logger
     * @param \Magento\Customer\Model\GroupRegistry $groupRegistry
     * @param \Magento\Framework\App\State $state
     * @param \Magento\Framework\Module\ModuleList\Loader $loader
     * @param \Magento\Config\Model\ResourceModel\Config $config
     * @param \Mailchimp $api
     * @param \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList
     * @param \Magento\Customer\Model\ResourceModel\CustomerRepositoryFactory $customerRepositoryFactory
     * @param \Ebizmarts\MailChimp\Model\MailChimpErrors $mailChimpErrors
     * @param \Ebizmarts\MailChimp\Model\MailChimpSyncEcommerceFactory $mailChimpSyncEcommerce
     * @param \Ebizmarts\MailChimp\Model\MailChimpSyncEcommerce $mailChimpSyncE
     * @param \Ebizmarts\MailChimp\Model\MailChimpSyncBatches $syncBatches
     * @param \Ebizmarts\MailChimp\Model\MailChimpStoresFactory $mailChimpStoresFactory
     * @param \Ebizmarts\MailChimp\Model\MailChimpStores $mailChimpStores
     * @param \Magento\Customer\Model\ResourceModel\Attribute\CollectionFactory $attCollection
     * @param \Magento\Framework\Encryption\Encryptor $encryptor
     * @param \Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory $subscriberCollection
     * @param \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerCollection
     * @param \Magento\Customer\Model\CustomerFactory $customerFactory
     * @param \Magento\Directory\Api\CountryInformationAcquirerInterfaceFactory $countryInformationFactory
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Ebizmarts\MailChimp\Model\MailChimpInterestGroupFactory $interestGroupFactory
     * @param \Magento\Framework\Stdlib\DateTime\DateTime $date
     * @param \FiloBlu\Newsletter\Model\SubscriberInfoFactory $subscriberInfoFactory
     * @param \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory
     * @param \FiloBlu\ExtEbizmartsMailChimp\Model\Logger\LoggerImport $loggerImport
     * @param \Magento\Customer\Api\GroupRepositoryInterface $groupRepository
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Ebizmarts\MailChimp\Model\Logger\Logger $logger,
        \Magento\Customer\Model\GroupRegistry $groupRegistry,
        \Magento\Framework\App\State $state,
        \Magento\Framework\Module\ModuleList\Loader $loader,
        \Magento\Config\Model\ResourceModel\Config $config,
        \Mailchimp $api,
        \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList,
        \Magento\Customer\Model\ResourceModel\CustomerRepositoryFactory $customerRepositoryFactory,
        \Ebizmarts\MailChimp\Model\MailChimpErrors $mailChimpErrors,
        \Ebizmarts\MailChimp\Model\MailChimpSyncEcommerceFactory $mailChimpSyncEcommerce,
        \Ebizmarts\MailChimp\Model\MailChimpSyncEcommerce $mailChimpSyncE,
        \Ebizmarts\MailChimp\Model\MailChimpSyncBatches $syncBatches,
        \Ebizmarts\MailChimp\Model\MailChimpStoresFactory $mailChimpStoresFactory,
        \Ebizmarts\MailChimp\Model\MailChimpStores $mailChimpStores,
        \Magento\Customer\Model\ResourceModel\Attribute\CollectionFactory $attCollection,
        \Magento\Framework\Encryption\Encryptor $encryptor,
        \Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory $subscriberCollection,
        \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerCollection,
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        \Magento\Directory\Api\CountryInformationAcquirerInterfaceFactory $countryInformationFactory,
        \Magento\Framework\App\ResourceConnection $resource,
        \Ebizmarts\MailChimp\Model\MailChimpInterestGroupFactory $interestGroupFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \FiloBlu\Newsletter\Model\SubscriberInfoFactory $subscriberInfoFactory,
        \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory,
        \FiloBlu\ExtEbizmartsMailChimp\Model\Logger\LoggerImport $loggerImport,
        \Magento\Customer\Api\GroupRepositoryInterface $groupRepository
    )
    {
        $this->_customerRepositoryFactory = $customerRepositoryFactory;
        $this->_storeManager = $storeManager;
        $this->_subscriberInfoFactory = $subscriberInfoFactory;
        $this->_subscriberFactory = $subscriberFactory;
        $this->_loggerImport = $loggerImport;
        $this->_groupRepository = $groupRepository;
        parent::__construct($context, $storeManager, $logger, $groupRegistry, $state, $loader, $config, $api, $cacheTypeList, $mailChimpErrors, $mailChimpSyncEcommerce, $mailChimpSyncE, $syncBatches, $mailChimpStoresFactory, $mailChimpStores, $attCollection, $encryptor, $subscriberCollection, $customerCollection, $customerFactory, $countryInformationFactory, $resource, $interestGroupFactory, $date);

    }

    /**
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function isCronEnabled()
    {
        return $this->getConfigValue(self::XML_PATH_CRON);
    }

    /**
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function isCronSyncStatusEnabled()
    {
        return $this->getConfigValue(self::XML_PATH_CRON_SYNC_STATUS);
    }

    /**
     * @param $subscribersIds
     * @return int
     */
    public function sendToMailchimp($subscribersIds)
    {

        $status = '';
        $count = 0;

        $countOk = 0;
        $countStatusNotAllowed = 0;
        $countError = 0;
        $this->_loggerImport->info(" #### START REQUEST COUNT -> " . count($subscribersIds) . " #### ");
        foreach ($subscribersIds as $subscriberId) {

            $this->_loggerImport->info("processing subscriber id " . $subscriberId);

            $subscriber = $this->_subscriberFactory->create()->load($subscriberId);

            if ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED || $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED) {

                if ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                    $status = 'subscribed';
                } else {
                    $status = 'unsubscribed';
                }

                $email = $subscriber->getSubscriberEmail();

                $this->_loggerImport->info("processing email " . $email);

                $mergeVars = $this->getMergeVarsSubscriber($subscriber, false, true);

//                if ($mergeVars) {
//
//                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
//                } else {
//                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
//                }

                try {
                    $md5HashEmail = md5(strtolower($email));
                    $api = $this->getApi($subscriber->getStoreId());
                    if ($subscriber->getMailchimpId()) {

                        $this->_loggerImport->info("update subscriber id " . $subscriberId);
                        $return = $api->lists->members->addOrUpdate($this->getDefaultList($subscriber->getStoreId()), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);
                    } else {
                        $this->_loggerImport->info("new subscriber id " . $subscriberId);
                        $return = $api->lists->members->addOrUpdate($this->getDefaultList($subscriber->getStoreId()), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);

                        if (isset($return["id"])) {
                            $subscriber->setMailchimpId($return["id"]);
                            $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp id " . $return["id"]);
                        } else {
                            $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp none");
                        }
                    }

                    $subscriber->save();

                    $countOk++;
                    $count++;
                } catch (\Exception $e) {
                    $this->_loggerImport->info($e->getMessage());
                    $countError++;
//                    $this->log($e->getMessage());
//                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {
//                        throw new \Exception($e->getMessage());
//                    }
                }
            } else {
                $this->_loggerImport->info("status not allowed for subscriber id" . $subscriberId);
                $countStatusNotAllowed++;
            }
        }

        $this->_loggerImport->info(" #### END REQUEST - Result Ok: " . $countOk . " / Result Error: " . $countError . " / Result NotAllowed: " . $countStatusNotAllowed . " #### ");

        return $count;
    }

    /**
     * @param $subscriber
     * @param bool $isCustomer
     * @param bool $loadFromTable
     * @return array
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getMergeVarsSubscriber($subscriber, $isCustomer = false, $loadFromTable = false)
    {
        $customerRepository = $this->_customerRepositoryFactory->create();
        $email = $subscriber->getEmail();

        if ($isCustomer) {
            $subscriberType = "CUSTOMER";
        } else {
            $subscriberType = "GUEST";
            try {
                $customerRepository->get($email, $this->_storeManager->getStore($subscriber->getStoreId())->getWebsite()->getId());
                $subscriberType = "CUSTOMER";
            } catch (\Exception $ex) {

                // No customer found
            }
        }

        $merge_vars = [];

        $mergeVars = unserialize($this->getConfigValue(self::XML_PATH_MAPPING));

        if ($loadFromTable) {
            $dataMerge = $this->_subscriberInfoFactory->create()->load($subscriber->getId())->getData();
        } else {
            $dataMerge = $this->_request->getPost();
        }

        $customer_group_name = $this->getCustomerGroupNameFromNewsletterFormData($dataMerge);

        foreach ($mergeVars as $map) {
            $merge_vars = array_merge($merge_vars, $this->_getMergeVarsValue($map, $dataMerge, $loadFromTable));
        }

        $merge_vars["TYPE"] = $subscriberType;
        if ($customer_group_name) {
            $merge_vars["CGROUP"] = $customer_group_name;
        }

        if ($subscriber->getMailchimpId()) {
            $merge_vars["CHIMPID"] = $subscriber->getMailchimpId();
        }

        return $merge_vars;
    }

    /**
     * @param $dataMerge
     * @return string|null
     */
    protected function getCustomerGroupNameFromNewsletterFormData($dataMerge)
    {
        $customerRepository = $this->_customerRepositoryFactory->create();
        // First search the websited ID using the website_name
        $websites = $this->_storeManager->getWebsites();
        $website_id = null;
        foreach ($websites as $w) {
            if ($w->getName() == $dataMerge['website_name']) {
                $website_id = $w->getWebsiteId();
                break;
            }
        }

        if (!$website_id) {
            return null;
        }

        try {
            $customer = $customerRepository->get($dataMerge['email'], $website_id);
        } catch (\Exception $ex) {
            return null;
        }

        return $this->getCustomerGroupName($customer);
    }

    /**
     * @param null $customer_obj
     * @return string|null
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCustomerGroupName($customer_obj = null)
    {
        if (!$customer_obj) {
            return null;
        }
        if (!is_numeric($customer_obj->getGroupId())) {
            return null;
        }
        $group = $this->_groupRepository->getById($customer_obj->getGroupId());
        if ($group && $group->getCode()) {
            return $group->getCode();
        }
        return null;
    }

    /**
     * @param $map
     * @param $dataMerge
     * @param bool $loadFromTable
     * @return array
     */
    protected function _getMergeVarsValue($map, $dataMerge, $loadFromTable = false)
    {
        $merge_vars = array();

        if ($loadFromTable) {
            $customAtt = $map['dbcolumnname'];
        } else {
            $customAtt = $map['magento'];
        }

        $chimpTag = $map['mailchimp'];

        if ($chimpTag && $customAtt) {
            $key = strtoupper($chimpTag);

            if (isset($dataMerge[$customAtt])) {
                if (is_array($dataMerge[$customAtt])) {
                    $valStr = "";
                    foreach ($dataMerge[$customAtt] as $val) {
                        if (is_string($val)) {
                            if ($valStr) {
                                $valStr .= ' | ' . $val;
                            } else {
                                $valStr = $val;
                            }
                        }
                    }

                    if ($valStr) {
                        $merge_vars[$key] = $valStr;
                    }
                } else {
                    if ($dataMerge[$customAtt]) {
                        $merge_vars[$key] = $dataMerge[$customAtt];
                    }
                }
            }

            return $merge_vars;
        }
    }

    /**
     * @param $subscribersIds
     * @return int
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function sendToMailchimpOnlyNew($subscribersIds)
    {

        $status = '';
        $count = 0;

        $countOk = 0;
        $countError = 0;

        $this->_loggerImport->info(" #### START REQUEST ONLY NEW COUNT -> " . count($subscribersIds) . " #### ");
        foreach ($subscribersIds as $subscriberId) {

            $subscriber = $this->_subscriberFactory->create()->load($subscriberId);

            if (!$subscriber->getMailchimpId() && ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED || $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED)) {

                $this->_loggerImport->info("processing subscriber id " . $subscriberId);

                if ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                    $status = 'subscribed';
                } else {
                    $status = 'unsubscribed';
                }

                $email = $subscriber->getSubscriberEmail();

                $this->_loggerImport->info("processing email " . $email);

                $mergeVars = $this->getMergeVarsSubscriber($subscriber, false, true);

//                if ($mergeVars) {
//                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status, 'merge_fields' => $mergeVars);
//                } else {
//                    $data = array('list_id' => $this->getDefaultList($subscriber->getStoreId()), 'email_address' => $email, 'email_type' => 'html', 'status' => $status);
//                }

                try {

                    $api = $this->getApi($subscriber->getStoreId());

                    $md5HashEmail = md5(strtolower($email));

                    $this->_loggerImport->info("new subscriber id " . $subscriberId);
                    $return = $api->lists->members->addOrUpdate($this->getDefaultList($subscriber->getStoreId()), $md5HashEmail, null, $status, $mergeVars, null, null, null, null, $email, $status);

                    if (isset($return["id"])) {
                        $subscriber->setMailchimpId($return["id"]);
                        $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp id " . $return["id"]);
                    } else {
                        $this->_loggerImport->info("subscriber id " . $subscriberId . " / mailchimp none");
                    }


                    $subscriber->save();

                    $countOk++;
                    $count++;
                } catch (\Exception $e) {
                    $this->_loggerImport->info($e->getMessage());
                    $countError++;
//                    $this->log($e->getMessage());
//                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {
//                        throw new \Exception($e->getMessage());
//                    }
                }
            }
        }

        $this->_loggerImport->info(" #### END REQUEST - Result Ok: " . $countOk . " / Result Error: " . $countError . " #### ");

        return $count;
    }


    /**
     * I need this because the $subscriber var from the getMergeVars function seems to be always null
     * This function will return the customer group name from the newsletter subscriber form (POST) values
     * @param $subscribersIds
     * @return int
     */
    public function updateMailChimpIdFromMageMonkeyId($subscribersIds)
    {

        $count = 0;
        $countOk = 0;
        $countError = 0;

        $this->_loggerImport->info(" #### START REQUEST UPDATE MAILCHIMP ID FROM MAGEMONKEY ID COUNT -> " . count($subscribersIds) . " #### ");
        foreach ($subscribersIds as $subscriberId) {

            $subscriber = $this->_subscriberFactory->create()->load($subscriberId);

            if (!$subscriber->getMailchimpId() && $subscriber->getMagemonkeyId()) {

                $this->_loggerImport->info("processing subscriber id " . $subscriberId);

                $email = $subscriber->getSubscriberEmail();

                $this->_loggerImport->info("processing email " . $email);

                try {

                    $subscriber->setMailchimpId($subscriber->getMagemonkeyId());
                    $subscriber->setMagemonkeyId('');

                    $subscriber->save();

                    $countOk++;
                    $count++;
                } catch (\Exception $e) {
                    $this->_loggerImport->info($e->getMessage());
                    $countError++;
                }
            }
        }

        $this->_loggerImport->info(" #### END REQUEST - Result Ok: " . $countOk . " / Result Error: " . $countError . " #### ");

        return $count;
    }

    /**
     * @param $subscribersIds
     * @return int
     */
    public function syncStatusFromMailChimpToMagento($subscribersIds)
    {

        $count = 0;

        $countOk = 0;
        $countError = 0;
        $countSkipped = 0;
        $this->_loggerImport->info(" #### START REQUEST SYNC MAILCHIMP TO MAGENTO COUNT -> " . count($subscribersIds) . " #### ");
        foreach ($subscribersIds as $subscriberId) {

            $this->_loggerImport->info("processing subscriber id " . $subscriberId);

            $subscriber = $this->_subscriberFactory->create()->load($subscriberId);

            if ($subscriber->getMailchimpId()) {

                $email = $subscriber->getSubscriberEmail();

                $this->_loggerImport->info("processing email " . $email);

                try {

                    $api = $this->getApi($subscriber->getStoreId());

                    $md5HashEmail = md5(strtolower($email));

                    $return = $api->lists->members->get($this->getDefaultList($subscriber->getStoreId()), $md5HashEmail);

                    if ($return["status"] == "subscribed" && $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                        $countSkipped++;
                        continue;
                    }

                    if ($return["status"] == "unsubscribed" && $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED) {
                        $countSkipped++;
                        continue;
                    }

                    // Subscribe to magento
                    if ($return["status"] == "subscribed" && $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED) {
                        $subscriber->setStatus(\Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED);
                        $this->_loggerImport->info($email . " changed status from unsubscribed to subscribed");
                        $subscriber->save();
                        $countOk++;
                        $count++;
                        continue;
                    }

                    // Unsubscribed to magento
                    if ($return["status"] == "unsubscribed" && $subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                        $subscriber->setStatus(\Magento\Newsletter\Model\Subscriber::STATUS_UNSUBSCRIBED);
                        $this->_loggerImport->info($email . " changed status from subscribed to unsubscribed");
                        $subscriber->save();
                        $countOk++;
                        $count++;
                        continue;
                    }

                    $countSkipped++;
//                    $count++;
                } catch (\Exception $e) {

                    $this->_loggerImport->info($e->getMessage());
                    $countError++;
//                    $this->log($e->getMessage());
//                    if (stripos($e->getMessage(), ' Status: 400 ') === false) {
//                        throw new \Exception($e->getMessage());
//                    }
                }
            } else {
                $countSkipped++;
            }
        }

        $this->_loggerImport->info(" #### END REQUEST - Result Ok: " . $countOk . " / Result Error: " . $countError . " / Result Skipped: " . $countSkipped . " #### ");

        return $count;
    }

}
